<?php
namespace LWS\Adminpanel\Pages\Field;

if (!defined('ABSPATH')) {
    exit();
}


/** Designed to be used inside Wizard only.
 * Behavior is similar to a radio,
 * But choices looks like tiles with a grid layout. */
class RadioGrid extends \LWS\Adminpanel\Pages\Field
{

	/** @return field html. */
	public static function compose($id, $extra=null)
	{
		$me = new self($id, '', $extra);
		return $me->html();
	}

	public function input()
	{
		echo $this->html();
	}

	public function html()
	{
		\wp_enqueue_script('lws-adm-radiogrid', LWS_ADMIN_PANEL_JS.'/controls/radiogrid.js', array('jquery'), LWS_ADMIN_PANEL_VERSION, true);
		\wp_enqueue_style('lws-adm-radiogrid', LWS_ADMIN_PANEL_CSS.'/controls/radiogrid.css', array(), LWS_ADMIN_PANEL_VERSION);

		$name = \esc_attr($this->id());
		$value = $this->readOption(true);
		$source = $this->getExtraValue('source', array());
		$html = "<input id='{$name}' name='{$name}' value='{$value}' type='hidden'>";

		if( 'large' == $this->getExtraValue('type') )
		{
			foreach( $source as $opt )
			{
				$opt = array_merge(
					array(
						'value'    => '',
						'img'      => '',
						'stars'    => '',
						'time'     => '',
						'pro-only' => false,
						'texts'    => array(),
						'labels'   => array(),
					),
					$opt
				);
				$opt['texts'] = array_merge(
					array(
						'title',
						'descr',
					),
					$opt['texts']
				);
				$selected = ($opt['value'] == $value ? ' selected' : '');
				foreach( array('value', 'img') as $attr )
					$opt[$attr] = \esc_attr($opt[$attr]);

				$bottom = '';
				if( \boolval($opt['pro-only']) )
				{
					$bottom = "<div class='lws-wizard-large-pro-only'>{$opt['labels']['pro']}</div>";
				}
				else
				{
					if( isset($opt['labels']['diff']) )
						$bottom .= "<div class='lws-wizard-large-diff-title'>{$opt['labels']['diff']}</div>";
					if( isset($opt['labels']['time']) )
						$bottom .= "<div class='lws-wizard-large-time-title'>{$opt['labels']['time']}</div>";

					$diff = '';
					if( isset($opt['texts']['diff']) )
						$diff .= "<div class='lws-wizard-large-diff-text'>{$opt['texts']['diff']}</div>";
					if( $opt['stars'] && ($opt['stars'] = \esc_attr($opt['stars'])) )
						$diff .= "<div class='lws-wizard-large-diff-stars'><img src='{$opt['stars']}' /></div>";
					if( $diff )
						$bottom .= "<div class='lws-wizard-large-diff-grid'>{$diff}</div>";

					$time = '';
					if( $opt['time'] )
						$time .= "<div class='lws-wizard-large-time-number'>{$opt['time']}</div>";
					if( isset($opt['labels']['unit']) )
						$time .= "<div class='lws-wizard-large-time-unit'>{$opt['labels']['unit']}</div>";
					if( $time )
						$bottom .= "<div class='lws-wizard-large-time-grid'>{$time}</div>";
				}

				$html .=<<<EOT
	<div class="lws-wizard-large-container lws_radiobutton_radio{$selected}" data-input='#{$name}' data-value='{$opt['value']}'>
		<div class="lws-wizard-large-image"><img src='{$opt['img']}'/></div>
		<div class="lws-wizard-large-desc-grid">
			<div class="lws-wizard-large-title">{$opt['texts']['title']}</div>
			<div class="lws-wizard-large-desc">{$opt['texts']['descr']}</div>
			{$bottom}
		</div>
	</div>
EOT;
			}
		}
		else
		{
			foreach( $source as $opt )
			{
				$opt = array_merge(
					array(
						'value'    => '',
						'class'    => '',
						'icon'     => '',
						'label'    => '',
					),
					$opt
				);
				$selected = ($opt['value'] == $value ? ' selected' : '');
				foreach( array('value', 'class', 'icon') as $attr )
					$opt[$attr] = \esc_attr($opt[$attr]);

				if( $opt['icon'] )
				{
					$html .= <<<EOT
<div class='lws-radiogrid-button {$opt['class']} lws_radiobutton_radio{$selected} ' data-input='#{$name}' data-value='{$opt['value']}'>
	<div class='lws-radiogrid-button-icon {$opt['icon']}'></div>
	<div class='lws-radiogrid-button-label'>{$opt['label']}</div>
</div>
EOT;
				}
				else
				{
					$html .= <<<EOT
<div class='lws-radiogrid-button no-icon {$opt['class']} lws_radiobutton_radio{$selected} ' data-input='#{$name}' data-value='{$opt['value']}'>
	<div class='lws-radiogrid-button-label'>{$opt['label']}</div>
</div>
EOT;
				}
			}
		}
		return $html;
	}
}
