<?php
namespace LWS\WOOREWARDS\Ui;

// don't call the file directly
if( !defined( 'ABSPATH' ) ) exit();

/** Create the backend menu and settings pages. */
class Admin
{
	public function __construct()
	{
		/** @param array, the fields settings array. @param Pool */
		\add_filter('lws_woorewards_admin_pool_general_settings', array($this, 'getPoolGeneralSettings'), 10, 2);

		lws_register_pages($this->getPages());
		\add_action('admin_enqueue_scripts', array($this , 'scripts'));

		// replace usual notice by a badge teaser
		if( !defined('LWS_WOOREWARDS_ACTIVATED') || !LWS_WOOREWARDS_ACTIVATED )
			\add_filter('pre_set_transient_settings_errors', array($this, 'noticeSettingsSaved'));

		$this->checkCouponsEnabled();
	}

	protected function checkCouponsEnabled()
	{
		if( defined('DOING_AJAX') && DOING_AJAX )
			return;

		if( function_exists('\wc_coupons_enabled') && !\wc_coupons_enabled() && !\get_option('lws_woorewards_ignore_woocommerce_disable_coupons') )
		{
			$ignore = false;
			if( isset($_GET['lws_wr_wc_coupons_enable']) && in_array($_GET['lws_wr_wc_coupons_enable'], array('yes', 'ignore')) )
			{
				if( $_GET['lws_wr_wc_coupons_enable'] == 'yes' )
				{
					\update_option('woocommerce_enable_coupons', 'yes');
					\update_option('lws_woorewards_ignore_woocommerce_disable_coupons', '');
				}
				else
					\update_option('lws_woorewards_ignore_woocommerce_disable_coupons', 'yes');
				$ignore = true;
			}

			if( !$ignore )
			{
				$message = array(
					__("WooCommerce Coupons are disabled. Several WooRewards features will be broken without Coupons.", 'woorewards-lite'),
					__("You can check your WooCommerce General Settings and look for : <b>Enable coupons</b>.", 'woorewards-lite'),
					sprintf(
						__('%1$s or %2$s this warning at your own risks.', 'woorewards-lite'),
						sprintf(
							"<a href='%s' class='button primary'>%s</a>",
							\esc_attr(\add_query_arg('lws_wr_wc_coupons_enable', 'yes')),
							__("Click here to resolve the problem immediately", 'woorewards-lite')
						),
						sprintf(
							"<a href='%s' class=''>%s</a>",
							\esc_attr(\add_query_arg('lws_wr_wc_coupons_enable', 'ignore')),
							__("ignore", 'woorewards-lite')
						)
					),
				);
				\lws_admin_add_notice_once('woocommerce_enable_coupons', implode('<br/>', $message), array('level'=>'error'));
			}
		}
	}

	public function scripts($hook)
	{
		// Force the menu icon with lws-icons font
		\wp_enqueue_style('wr-menu-icon', LWS_WOOREWARDS_CSS . '/menu-icon.css', array(), LWS_WOOREWARDS_VERSION);

		if( false !== ($ppos = strpos($hook, LWS_WOOREWARDS_PAGE)) )
		{
			$page = substr($hook, $ppos);
			$tab = isset($_GET['tab']) ? $_GET['tab'] : '';

			if( !defined('LWS_WOOREWARDS_ACTIVATED') || !LWS_WOOREWARDS_ACTIVATED )
			{
				// let badge teaser replace the notice
				\wp_enqueue_style('lws-wre-notice', LWS_WOOREWARDS_CSS . '/notice.css', array(), LWS_WOOREWARDS_VERSION);
			}

			if( ($page == LWS_WOOREWARDS_PAGE || false !== strpos($page, 'customers')) && (empty($tab) || strpos($tab, 'wr_customers') !== false) )
			{
				// labels displayed in points history
				$labels = array(
					'hist' => __("Points History", 'woorewards-lite'),
					'desc' => __("Description", 'woorewards-lite'),
					'date' => __("Date", 'woorewards-lite'),
					'points' => __("Points", 'woorewards-lite'),
					'total' => __("Total", 'woorewards-lite'),
				);
				// enqueue editlist column folding script
				foreach( ($deps = array('jquery', 'lws-tools')) as $dep )
					\wp_enqueue_script($dep);

				\wp_register_script('lws-wre-userspoints', LWS_WOOREWARDS_JS . '/userspoints.js', $deps, LWS_WOOREWARDS_VERSION, true);
				\wp_localize_script('lws-wre-userspoints', 'lws_wr_userspoints_labels', $labels);
				\wp_enqueue_script('lws-wre-userspoints');
				\wp_enqueue_style('lws-wre-userspoints', LWS_WOOREWARDS_CSS . '/userspoints.css', array(), LWS_WOOREWARDS_VERSION);

				\do_action('lws_adminpanel_enqueue_lac_scripts', array('select'));
				\do_action('lws_woorewards_ui_userspoints_enqueue_scripts', $hook, $tab);
			}
			else if( false !== strpos($page, 'loyalty') )
			{
				foreach( ($deps = array('jquery')) as $dep )
					\wp_enqueue_script($dep);
				\wp_enqueue_script('lws-wre-poolsettings', LWS_WOOREWARDS_JS . '/poolsettings.js', $deps, LWS_WOOREWARDS_VERSION, true);
				\wp_enqueue_style('lws-wre-poolsettings', LWS_WOOREWARDS_CSS . '/poolsettings.css', array(), LWS_WOOREWARDS_VERSION);

				\do_action('lws_adminpanel_enqueue_lac_scripts', array('select'));
				\wp_enqueue_script('lws-checkbox');
				\wp_enqueue_style('lws-checkbox');
				\wp_enqueue_script('lws-switch');
				\wp_enqueue_style('lws-switch');
				\wp_enqueue_script('lws-spinner');
				\wp_enqueue_style('lws-spinner');
			}

			\wp_enqueue_script('lws-wre-coupon-edit', LWS_WOOREWARDS_JS . '/couponedit.js', array('jquery'), LWS_WOOREWARDS_VERSION, true);
		}
	}

	/** Push an achievement teaser instead our usual notice at setting save. */
	public function noticeSettingsSaved($value)
	{
		if( !empty($value) && isset($_POST['option_page']) && false !== strpos($_POST['option_page'], LWS_WOOREWARDS_PAGE) )
		{
			$val = \current($value);
			if( isset($val['type']) && $val['type'] == 'updated' && isset($val['code']) && $val['code'] == 'settings_updated' )
			{
				$teasers = array(
					__("Add fun and achievements for your customers with the <a>Pro Version</a>", 'woorewards-lite'),
					__("Try the <a>Pro Version</a> for free for 30 days", 'woorewards-lite'),
					__("The <a>Pro Version</a> adds Events and Levelling systems. Try <a>it</a>", 'woorewards-lite')
				);
				\LWS_WooRewards::achievement(array(
					'title'   => __("Your settings have been saved.", 'woorewards-lite'),
					'message' => str_replace(
						'<a>',
						"<a href='https://plugins.longwatchstudio.com/product/woorewards/' target='_blank'>",
						$teasers[rand(0, count($teasers)-1)]
					)
				));
			}
		}
		return $value;
	}

	protected function getPages()
	{
		$pages = $this->getMainMenu();

		$first = array_keys($pages)[0];
		$pages[$first] = array_merge($pages[$first], array(
			'id'        => LWS_WOOREWARDS_PAGE,
			'dashicons' => '',
			'index'     => 57,
		));

		$title = __("WooRewards", 'woorewards-lite');
		foreach( $pages as &$p )
		{
			$p['subtitle'] = $p['title'];
			$p['title'] = $title;
		}

		return $pages;
	}

	protected function getMainMenu()
	{
		if( false === ($customer = \apply_filters('lws_woorewards_ui_customers_tab_get', false)) )
		{
			$customer = array(
				'title'    => __("Customers", 'woorewards-lite'),
				'id'       => LWS_WOOREWARDS_PAGE.'.customers',
				'rights'   => 'manage_rewards',
				'tabs'     => array('wr_customers' => array(
					'title'    => __("Customers", 'woorewards-lite'),
					'id'       => 'wr_customers',
					'groups'   => $this->getCustomersGroups()
				))
			);
		}

		if( false === ($loyalty = \apply_filters('lws_woorewards_ui_loyalty_tab_get', false)) )
		{
			$loyalty = array(
				'title'    => __("Loyalty System", 'woorewards-lite'),
				'rights'   => 'manage_rewards',
				'id'       => LWS_WOOREWARDS_PAGE.'.loyalty',
				'tabs' => array('wr_loyalty' => array(
					'title'    => __("Loyalty System", 'woorewards-lite'),
					'id'       => 'wr_loyalty',
					'groups'   => $this->getLoyaltyGroups()
				))
			);
		}

		$pages = array(
			'wr_customers' => $customer,
			'wr_loyalty'   => $loyalty,
			'wr_settings'  => array(
				'title'    => __("Settings", 'woorewards-lite'),
				'subtitle' => __("Settings", 'woorewards-lite'),
				'id'       => LWS_WOOREWARDS_PAGE.'.settings',
				'rights'   => 'manage_rewards',
				'tabs'     => $this->getSettingsTabs()
			),
		);
		if( defined('LWS_WIZARD_SUMMONER') )
		{
			$pages['wr_wizard'] = array(
				'title'    => __("Wizard", 'woorewards-lite'),
				'subtitle' => __("Wizard", 'woorewards-lite'),
				'id'       => LWS_WIZARD_SUMMONER.LWS_WOOREWARDS_PAGE,
				'rights'   => 'manage_rewards',
			);
		}

		return $pages;
	}

	/** For all users, list points for each pool. */
	protected function getCustomersGroups()
	{
		require_once LWS_WOOREWARDS_INCLUDES . '/ui/editlists/userspoints.php';
		require_once LWS_WOOREWARDS_INCLUDES . '/ui/editlists/userspointsbulkaction.php';
		$editlist = \lws_editlist(
			'userspoints',
			'user_id',
			new \LWS\WOOREWARDS\Ui\Editlists\UsersPoints(),
			\LWS\Adminpanel\EditList::FIX,
			\apply_filters('lws_woorewards_ui_userspoints_filters', array(
				'user_search' => new \LWS\Adminpanel\EditList\FilterSimpleField('usersearch', __('Search...', 'woorewards-lite')),
				'points_add'  => new \LWS\WOOREWARDS\Ui\Editlists\UsersPointsBulkAction('points_add')
			))
		);

		$customersGroup = array(
			'customers_points' => array(
				'title'    => __("Points Management", 'woorewards-lite'),
				'text'     => __("Here you can see and manage your customers reward points", 'woorewards-lite')
					."<br/>".__("You can view the points <b>history</b> by clicking the points total in the table", 'woorewards-lite')
					."<br/>".sprintf(__("You could present a point history to your customer by using the shortcode %s", 'woorewards-lite'),'<b>[wr_show_history]</b>'),
				'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards/customers-management/'),
				'editlist' => $editlist,
			)
		);

		return $customersGroup;

	}

	/** Tease about pro version.
	 * Display standand pool settings. */
	protected function getLoyaltyGroups()
	{
		$groups = array();

		if( !\LWS_WooRewards::isWC() && (!defined('LWS_WOOREWARDS_ACTIVATED') || !LWS_WOOREWARDS_ACTIVATED) )
		{
			$groups['information'] = array(
				'id'    => 'information',
				'title' => __("Information", 'woorewards-lite'),
				'text'  => __(
"WooRewards Standard uses WooCommerce <i>orders</i> and <i>coupons</i>.
<br/>You should install <a href='https://wordpress.org/plugins/woocommerce/' target='_blank'>WooCommerce</a> to have them active.
<br/>Or <a href='https://plugins.longwatchstudio.com/product/woorewards/' target='_blank'>upgrade <b>WooRewards</b> to the <b>Pro</b> version</a>
and enjoy new ways to earn points (social media, sponsoring... with or without WooCommerce) and a lot of new reward types !",
					'woorewards-lite')
			);
		}

		// load the default pool
		$pools = \LWS\WOOREWARDS\Collections\Pools::instanciate()->load(array(
			'meta_query'  => array(
				array(
					'key'     => 'wre_pool_prefab',
					'value'   => 'yes',
					'compare' => 'LIKE'
				),
				array(
					'key'     => 'wre_pool_type',
					'value'   => \LWS\WOOREWARDS\Core\Pool::T_STANDARD,
					'compare' => 'LIKE'
				)
			)
		));

		if( $pools->count() <= 0 )
		{
			if( defined('LWS_WIZARD_SUMMONER') && LWS_WIZARD_SUMMONER )
			{
				$url = \esc_attr(\add_query_arg(array('page'=>LWS_WIZARD_SUMMONER.LWS_WOOREWARDS_PAGE), admin_url('admin.php')));
				$attr = " href='{$url}' class='lws-adm-btn'";
				$text = sprintf(
					__("It seems the prefab pool does not exists. Please run the <a%s>wizard</a> to create one.", 'woorewards-lite'),
					$attr
				);
				\lws_admin_add_notice_once('lws-wre-pool-nothing-loaded', sprintf(
					__("No loyalty system available. Please run the <a%s>wizard</a> to install default one.", 'woorewards-lite'),
					$attr
				), array('level'=>'info', 'dismissible'=>true));
			}
			else
			{
				$text = __("It seems the prefab pool does not exists. Try to re-activate this plugin. If the problem persists, contact your administrator.", 'woorewards-lite');
				\lws_admin_add_notice_once('lws-wre-pool-nothing-loaded', __("No loyalty system available. Try to re-activate the WooRewards plugin to install default ones.", 'woorewards-lite'), array('level'=>'info', 'dismissible'=>true));
			}

			error_log("Warning: No pools loaded.");
			$groups['failure'] = array(
				'id'    => 'failure',
				'title' => __("Loading failure", 'woorewards-lite'),
				'text'  => $text,
			);
		}
		else
		{
			// let dedicated class create options
			$pool = $pools->get(0);
			$groups = array_merge($groups, array(
				'general'    => array(
					'id'       => 'general',
					'title'    => __("General Settings", 'woorewards-lite'),
					'fields'   => \apply_filters('lws_woorewards_admin_pool_general_settings', array(), $pool)
				),
				'earning'    => array(
					'id'       => 'earning',
					'title'    => __("Earning points", 'woorewards-lite'),
					'text'     => __("Here you can manage how your customers earn loyalty points", 'woorewards-lite'),
					'editlist' => \lws_editlist(
						'EventList',
						\LWS\WOOREWARDS\Ui\Editlists\EventList::ROW_ID,
						new \LWS\WOOREWARDS\Ui\Editlists\EventList($pool),
						\LWS\Adminpanel\EditList::MOD
					)->setPageDisplay(false)->setCssClass('eventlist')
				),
				'spending'   => array(
					'id'       => 'spending',
					'title'    => __("Rewards", 'woorewards-lite'),
					'text'     => __("Here you can manage the rewards for your customers", 'woorewards-lite'),
					'editlist' => \lws_editlist(
						'UnlockableList',
						\LWS\WOOREWARDS\Ui\Editlists\UnlockableList::ROW_ID,
						new \LWS\WOOREWARDS\Ui\Editlists\UnlockableList($pool),
						\LWS\Adminpanel\EditList::MOD
					)->setPageDisplay(false)->setCssClass('unlockablelist')
				)
			));
		}
		if( (!defined('LWS_WOOREWARDS_ACTIVATED') || !LWS_WOOREWARDS_ACTIVATED) )
		{
			if( isset($groups['earning']) )
			{
				$groups['earning']['fields']= array(
					'tutorial' => array(
						'id' => 'lws_woorewards_tutorial_earning_vfree',
						'type' => 'help',
						'extra' => array(
							'help' => __("Watch our <a target='_blank' href='https://youtu.be/yl9WyGOGd_I'>youtube tutorial</a> to help you set up your loyalty program", 'woorewards-lite'),
							'type' => 'youtube'
						)
					),
				);
			}
		}

		return $groups;
	}

	/** For pool option in admin page:
	 * *	be sure field id starts with 'lws-wr-pool-option-' and Pool->setOption accept the id string rest as valid option name.
	 * *	be sure the page contains a <input> named 'pool' with relevant pool id.
	 * *	since field cannot read value in wp get_option, be sure to set the relevant value in extra array.
	 *
	 *	@param fields an array as required by 'fields' entry in admin group.
	 * 	@param $pool a Pool instance. */
	public function getPoolGeneralSettings($fields, \LWS\WOOREWARDS\Core\Pool $pool)
	{
		$poolOptionPrefix = 'lws-wr-pool-option-';

		$fields['pool'] = array(
			'id'    => 'lws-wr-pool-option',
			'type'  => 'hidden',
			'extra' => array(
				'value' => $pool->getId()
			)
		);

		$fields['enabled'] = array(
			'id'    => $poolOptionPrefix.'enabled', /// id starts with 'lws-wr-pool-option-', 'enabled' is accepted as Pool option
			'type'  => 'box',
			'title' => 'Status',
			'extra' => array(
				'class'   => 'lws_switch lws-force-confirm',
				'checked' => $pool->getOption('enabled'), /// set field value here
				'data'    => array(
					'default' => _x("Off", "pool enabled switch", 'woorewards-lite'),
					'checked' => _x("On", "pool enabled switch", 'woorewards-lite')
				)
			)
		);

		$fields['title'] = array(
			'id'    => $poolOptionPrefix.'title',
			'type'  => 'text',
			'title' => _x("Title", "Pool title", 'woorewards-lite'),
			'extra' => array(
				'required' => true,
				'value'    => $pool->getOption('title')
			)
		);

		return $fields;
	}

	protected function getSettingsTabs()
	{
		$tabs = array(
			'settings' => array(
				'id'     => 'settings',
				'title'  => __("General settings", 'woorewards-lite'),
				'groups' => array(
					'settings' => array(
						'id'     => 'settings',
						'title'  => __("General settings", 'woorewards-lite'),
						'extra'    => array('doclink' => 'https://plugins.longwatchstudio.com/docs/woorewards/other-settings/general-settings/'),
						'fields' => array(
							'inc_taxes' => array(
								'id'    => 'lws_woorewards_order_amount_includes_taxes',
								'title' => __("Includes taxes", 'woorewards-lite'),
								'type'  => 'box',
								'extra' => array(
									'class' => 'lws_checkbox',
									'help'=>__("If checked, taxes will be included in the points earned when spending money", 'woorewards-lite'),
								)
							),
							'order_state' => array(
								'id'    => 'lws_woorewards_coupon_state',
								'title' => __("Points on 'Complete' order only", 'woorewards-lite'),
								'type'  => 'box',
								'extra' => array(
									'class' => 'lws_checkbox',
									'help' => __("Default state to get points is the processing order status.<br/>If you want to use the complete order status instead (recommanded), check the below box", 'woorewards-lite'),
								)
							)
						)
					),
					'wc_old_orders' => array(
						'id' => 'wc_old_orders',
						'title' => __("Give Points for Past orders", 'woorewards-lite'),
						'text' => __("If you want to give points for orders that pre-existed your loyalty system, you can do it here", 'woorewards-lite')
							. '<br/>' . __("This operation can take several minutes. Depending on you server configuration and date range, you should run this operation several times on short dates ranges.", 'woorewards-lite'),
						'fields' => array(
							'date_min' => array(
								'id'    => 'date_min',
								'title' => __("Start date", 'woorewards-lite'),
								'type'  => 'input',
								'extra' => array(
									'advanced' => true,
									'type'     =>'date',
									'gizmo'    => true,
									'class'    => 'lws-ignore-confirm',
									'default'  => \date_create()->sub(new \DateInterval('P1M'))->format('Y-m-d'),
								),
							),
							'date_max' => array(
								'id' => 'date_max',
								'title' => __("End date", 'woorewards-lite'),
								'type'  => 'input',
								'extra' => array(
									'advanced' => true,
									'type'     =>'date',
									'gizmo'    => true,
									'class'    => 'lws-ignore-confirm',
									'default'  => \date('Y-m-d'),
								),
							),
							'trigger_orders' => array(
								'id' => 'trigger_orders',
								'title' => __("Launch the procedure", 'woorewards-lite'),
								'type' => 'button',
								'extra' => array(
									'advanced' => true,
									'callback' => array($this, 'forceOldOrdersTrigger')
								),
							),
						)
					),
				)
			),
			'sty_mails' => array(
				'id'     => 'sty_mails',
				'title'  => __("Emails", 'woorewards-lite'),
				'groups' => \lws_mail_settings(\apply_filters('lws_woorewards_mails', array()))
			),
			'sty_widgets' => array(
				'id'     => 'sty_widgets',
				'title'  => __("Widgets", 'woorewards-lite'),
				'groups' => array(
					'showpoints' => array(
						'id' => 'showpoints',
						'title' => __("Display Points Widget", 'woorewards-lite'),
						'text' => '',
						'fields' => array(
							'spunconnected' => array(
								'id' => 'lws_wooreward_showpoints_nouser',
								'title' => __("Text displayed if user not connected", 'woorewards-lite'),
								'type' => 'text',
								'extra' => array(
									'size' => '50',
									'placeholder' => __("Please log in if you want to see your loyalty points", 'woorewards-lite'),
									'help' => __("In this Widget, customers can see their Reward Points available, as well as a link to the 'My Account' page to see more details.", 'woorewards-lite')."<br/>".
									sprintf(__("If you want, you can use the following shortcode instead : %s.", 'woorewards-lite'),"<span class='lws-group-descr-copy lws_ui_value_copy'><div class='lws-group-descr-copy-text content' tabindex='0'>[wr_show_points]</div><div class='lws-group-descr-copy-icon lws-icon lws-icon-copy copy'></div></span>")
								)
							),
							'showpoints' => array(
								'id' => 'lws_woorewards_displaypoints_template',
								'type' => 'stygen',
								'extra' => array(
									'purpose' => 'filter',
									'template' => 'wr_display_points',
									'html'=> false,
									'css'=>LWS_WOOREWARDS_URL.'/css/templates/displaypoints.css',
									'help' => __("Here you can customize the look and displayed text of the shortcode/widget", 'woorewards-lite'),
									'subids' => array(
										'lws_woorewards_displaypoints_title' => "WooRewards Show Points - title", // no translation on purpose
										'lws_woorewards_button_more_details' => "WooRewards Show Points - details", // no translation on purpose
									)
								)
							)
						)
					)
				)
			)
		);
		return $tabs;
	}

	/** Simulate the order status change for order in date range */
	function forceOldOrdersTrigger($btnId, $data=array())
	{
		if( $btnId != 'trigger_orders' ) return false;
		if( !isset($data['date_min']) || !($d1 = \date_create($data['date_min'])) ) return __("Dates are required", 'woorewards-lite');
		if( !isset($data['date_max']) || !($d2 = \date_create($data['date_max'])) ) return __("Dates are required", 'woorewards-lite');
		if( $d2 < $d1 )
		{
			$tmp = $d2;
			$d2 = $d1;
			$d1 = $tmp;
		}
		$d1 = $d1->format('Y-m-d');
		$d2 = $d2->format('Y-m-d');

		$status = array_unique(\apply_filters('lws_woorewards_order_events', array('processing', 'completed')));
		$status = array_map(function($s){return 'wc-'.$s;}, $status);
		$status = implode("','", array_map('\esc_sql', $status));

		$shopKind = \apply_filters('lws_woorewards_order_backward_apply_shop_kind', array('shop_order'));
		$shopKind = implode("','", array_map('\esc_sql', $shopKind));

		global $wpdb;
		$sql = <<<EOT
SELECT p.ID
FROM {$wpdb->posts} as p
WHERE p.post_type IN ('{$shopKind}')
AND p.post_status IN ('{$status}')
AND DATE(p.post_date) BETWEEN DATE('{$d1}') AND DATE('{$d2}')
GROUP BY p.ID
EOT;
		$orderIds = $wpdb->get_col($sql);
		if( false === $orderIds ) return false;

		$count = 0;
		foreach( $orderIds as $orderId )
		{
			if( $order = \wc_get_order($orderId) )
			{
				$hook = 'lws_woorewards_pool_on_order_done';
//				$hook = 'woocommerce_order_status_' . $order->get_status('edit'); // we test process unicity, but not sure any third party do it correctly
				\do_action($hook, $orderId, $order);
				++$count;
			}
		}

		return sprintf(__("<b>%s</b> order(s) processed.", 'woorewards-lite'), $count);
	}
}
